Module: Thor::Invocation::ClassMethods

Defined in:
lib/thor/invocation.rb

Instance Method Summary collapse

Instance Method Details

#prepare_for_invocation(key, name) ⇒ Object

Prepare for class methods invocations. This method must return a klass to have the invoked class options showed in help messages in generators.



11
12
13
14
15
16
17
18
# File 'lib/thor/invocation.rb', line 11

def prepare_for_invocation(key, name) #:nodoc:
  case name
    when Symbol, String
      Thor::Util.namespace_to_thor_class_and_task(name.to_s, false)
    else
      name
  end
end