Class: Thor::Shell::Basic

Inherits:
Object show all
Defined in:
lib/thor/shell/basic.rb

Direct Known Subclasses

Color

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initializeBasic

Initialize base and padding to nil.



10
11
12
# File 'lib/thor/shell/basic.rb', line 10

def initialize #:nodoc:
  @base, @padding = nil, 0
end

Instance Attribute Details

#baseObject

Returns the value of attribute base.



6
7
8
# File 'lib/thor/shell/basic.rb', line 6

def base
  @base
end

#paddingObject

Returns the value of attribute padding.



6
7
8
# File 'lib/thor/shell/basic.rb', line 6

def padding
  @padding
end

Instance Method Details

#ask(statement, color = nil) ⇒ Object

Ask something to the user and receives a response.

Example

ask("What is your name?")



25
26
27
28
# File 'lib/thor/shell/basic.rb', line 25

def ask(statement, color=nil)
  say("#{statement} ", color)
  $stdin.gets.strip
end

#error(statement) ⇒ Object

Called if something goes wrong during the execution. This is used by Thor internally and should not be used inside your scripts. If someone went wrong, you can always raise an exception. If you raise a Thor::Error, it will be rescued and wrapped in the method below.



170
171
172
# File 'lib/thor/shell/basic.rb', line 170

def error(statement)
  $stderr.puts statement
end

#file_collision(destination) ⇒ Object

Deals with file collision and returns true if the file should be overwriten and false otherwise. If a block is given, it uses the block response as the content for the diff.

Parameters

destination

the destination file to solve conflicts

block

an optional block that returns the value to be used in diff



139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
# File 'lib/thor/shell/basic.rb', line 139

def file_collision(destination)
  return true if @always_force
  options = block_given? ? "[Ynaqdh]" : "[Ynaqh]"

  while true
    answer = ask %[Overwrite #{destination}? (enter "h" for help) #{options}]

    case answer
      when is?(:yes), is?(:force)
        return true
      when is?(:no), is?(:skip)
        return false
      when is?(:always)
        return @always_force = true
      when is?(:quit)
        say 'Aborting...'
        raise SystemExit
      when is?(:diff)
        show_diff(destination, yield) if block_given?
        say 'Retrying...'
      else
        say file_collision_help
    end
  end
end

#no?(statement, color = nil) ⇒ Boolean

Make a question the to user and returns true if the user replies "n" or "no".

Returns:

  • (Boolean)


75
76
77
# File 'lib/thor/shell/basic.rb', line 75

def no?(statement, color=nil)
  !yes?(statement, color)
end

Prints a list of items.

Parameters

list<Array[String, String, ...]>

Options

mode

Can be :rows or :inline. Defaults to :rows.

ident

Ident each item with the value given.



88
89
90
91
92
93
94
95
96
97
98
99
100
101
# File 'lib/thor/shell/basic.rb', line 88

def print_list(list, options={})
  return if list.empty?

  ident   = " " * (options[:ident] || 0)
  content = case options[:mode]
    when :inline
      last = list.pop
      "#{list.join(", ")}, and #{last}"
    else # rows
      ident + list.join("\n#{ident}")
  end

  $stdout.puts content
end

Prints a table.

Parameters

Array[Array[String, String, ...]]

Options

ident

Ident the first column by ident value.



111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/thor/shell/basic.rb', line 111

def print_table(table, options={})
  return if table.empty?

  formats = []
  0.upto(table.first.length - 2) do |i|
    maxima = table.max{ |a,b| a[i].size <=> b[i].size }[i].size
    formats << "%-#{maxima + 2}s"
  end

  formats[0] = formats[0].insert(0, " " * options[:ident]) if options[:ident]
  formats << "%s"

  table.each do |row|
    row.each_with_index do |column, i|
      $stdout.print formats[i] % column.to_s
    end
    $stdout.puts
  end
end

#say(message = "", color = nil, force_new_line = false) ⇒ Object

Say (print) something to the user. If the sentence ends with a whitespace or tab character, a new line is not appended (print + flush). Otherwise are passed straight to puts (behavior got from Highline).

Example

say("I know you knew that.")



37
38
39
40
41
42
43
44
45
46
47
48
# File 'lib/thor/shell/basic.rb', line 37

def say(message="", color=nil, force_new_line=false)
  message  = message.to_s
  new_line = force_new_line || !(message[-1, 1] == " " || message[-1, 1] == "\t")
  message  = set_color(message, color) if color

  if new_line
    $stdout.puts(message)
  else
    $stdout.print(message)
    $stdout.flush
  end
end

#say_status(status, message, log_status = true) ⇒ Object

Say a status with the given color and appends the message. Since this method is used frequently by actions, it allows nil or false to be given in log_status, avoiding the message from being shown. If a Symbol is given in log_status, it's used as the color.



55
56
57
58
59
60
61
62
63
# File 'lib/thor/shell/basic.rb', line 55

def say_status(status, message, log_status=true)
  return if quiet? || log_status == false
  spaces = "  " * (padding + 1)
  color  = log_status.is_a?(Symbol) ? log_status : :green

  status = status.to_s.rjust(12)
  status = set_color status, color, true if color
  say "#{status}#{spaces}#{message}", nil, true
end

#set_color(string, color, bold = false) ⇒ Object

Apply color to the given string with optional bold. Disabled in the Thor::Shell::Basic class.



177
178
179
# File 'lib/thor/shell/basic.rb', line 177

def set_color(string, color, bold=false) #:nodoc:
  string
end

#yes?(statement, color = nil) ⇒ Boolean

Make a question the to user and returns true if the user replies "y" or "yes".

Returns:

  • (Boolean)


68
69
70
# File 'lib/thor/shell/basic.rb', line 68

def yes?(statement, color=nil)
  ask(statement, color) =~ is?(:yes)
end