Module: Thor::Base::ClassMethods
- Defined in:
- lib/thor/base.rb
Instance Method Summary collapse
-
#all_tasks ⇒ Object
Returns the tasks for this Thor class and all subclasses.
-
#argument(name, options = {}) ⇒ Object
Adds an argument to the class and creates an attr_accessor for it.
-
#arguments ⇒ Object
Returns this class arguments, looking up in the ancestors chain.
-
#class_option(name, options = {}) ⇒ Object
Adds an option to the set of class options.
-
#class_options(options = nil) ⇒ Object
Adds a bunch of options to the set of class options.
-
#group(name = nil) ⇒ Object
Defines the group.
-
#namespace(name = nil) ⇒ Object
Sets the namespace for the Thor or Thor::Group class.
-
#no_tasks ⇒ Object
All methods defined inside the given block are not added as tasks.
-
#remove_argument(*names) ⇒ Object
Removes a previous defined argument.
-
#remove_class_option(*names) ⇒ Object
Removes a previous defined class option.
-
#remove_task(*names) ⇒ Object
Removes a given task from this Thor class.
-
#start(given_args = ARGV, config = {}) ⇒ Object
Default way to start generators from the command line.
-
#tasks ⇒ Object
Returns the tasks for this Thor class.
Instance Method Details
#all_tasks ⇒ Object
Returns the tasks for this Thor class and all subclasses.
Returns
- OrderedHash
An ordered hash with tasks names as keys and Thor::Task objects as values.
264 265 266 267 |
# File 'lib/thor/base.rb', line 264 def all_tasks @all_tasks ||= from_superclass(:all_tasks, Thor::CoreExt::OrderedHash.new) @all_tasks.merge(tasks) end |
#argument(name, options = {}) ⇒ Object
Adds an argument to the class and creates an attr_accessor for it.
Arguments are different from options in several aspects. The first one is how they are parsed from the command line, arguments are retrieved from position:
thor task NAME
Instead of:
thor task --name=NAME
Besides, arguments are used inside your code as an accessor (self.argument), while options are all kept in a hash (self.options).
Finally, arguments cannot have type :default or :boolean but can be optional (supplying :optional => :true or :required => false), although you cannot have a required argument after a non-required argument. If you try it, an error is raised.
Parameters
- name
The name of the argument.
- options
Described below.
Options
:desc - Description for the argument. :required - If the argument is required or not. :optional - If the argument is optional or not. :type - The type of the argument, can be :string, :hash, :array, :numeric. :default - Default value for this argument. It cannot be required and have default values. :banner - String to show on usage notes.
Errors
- ArgumentError
Raised if you supply a required argument after a non required one.
130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 |
# File 'lib/thor/base.rb', line 130 def argument(name, ={}) is_thor_reserved_word?(name, :argument) no_tasks { attr_accessor name } required = if .key?(:optional) ![:optional] elsif .key?(:required) [:required] else [:default].nil? end remove_argument name arguments.each do |argument| next if argument.required? raise ArgumentError, "You cannot have #{name.to_s.inspect} as required argument after " << "the non-required argument #{argument.human_name.inspect}." end if required arguments << Thor::Argument.new(name, [:desc], required, [:type], [:default], [:banner]) end |
#arguments ⇒ Object
159 160 161 |
# File 'lib/thor/base.rb', line 159 def arguments @arguments ||= from_superclass(:arguments, []) end |
#class_option(name, options = {}) ⇒ Object
Adds an option to the set of class options
Parameters
- name
The name of the argument.
- options
Described below.
Options
:desc - Description for the argument. :required - If the argument is required or not. :default - Default value for this argument. :group - The group for this options. Use by class options to output options in different levels. :aliases - Aliases for this option. :type - The type of the argument, can be :string, :hash, :array, :numeric or :boolean. :banner - String to show on usage notes.
193 194 195 |
# File 'lib/thor/base.rb', line 193 def class_option(name, ={}) build_option(name, , ) end |
#class_options(options = nil) ⇒ Object
Adds a bunch of options to the set of class options.
:foo => false, :bar => :required, :baz => :string
If you prefer more detailed declaration, check class_option.
Parameters
Hash[Symbol => Object]
172 173 174 175 176 |
# File 'lib/thor/base.rb', line 172 def (=nil) @class_options ||= from_superclass(:class_options, {}) (, @class_options) if @class_options end |
#group(name = nil) ⇒ Object
Defines the group. This is used when thor list is invoked so you can specify that only tasks from a pre-defined group will be shown. Defaults to standard.
Parameters
name<String|Symbol>
239 240 241 242 243 244 245 246 |
# File 'lib/thor/base.rb', line 239 def group(name=nil) case name when nil @group ||= from_superclass(:group, 'standard') else @group = name.to_s end end |
#namespace(name = nil) ⇒ Object
Sets the namespace for the Thor or Thor::Group class. By default the namespace is retrieved from the class name. If your Thor class is named Scripts::MyScript, the help method, for example, will be called as:
thor scripts:my_script -h
If you change the namespace:
namespace :my_scripts
You change how your tasks are invoked:
thor my_scripts -h
Finally, if you change your namespace to default:
namespace :default
Your tasks can be invoked with a shortcut. Instead of:
thor :my_task
338 339 340 341 342 343 344 345 |
# File 'lib/thor/base.rb', line 338 def namespace(name=nil) case name when nil @namespace ||= Thor::Util.namespace_from_thor_class(self, false) else @namespace = name.to_s end end |
#no_tasks ⇒ Object
310 311 312 313 314 |
# File 'lib/thor/base.rb', line 310 def no_tasks @no_tasks = true yield @no_tasks = false end |
#remove_argument(*names) ⇒ Object
Removes a previous defined argument. If :undefine is given, undefine accessors as well.
Paremeters
- names
Arguments to be removed
Examples
remove_argument :foo
remove_argument :foo, :bar, :baz, :undefine => true
208 209 210 211 212 213 214 215 |
# File 'lib/thor/base.rb', line 208 def remove_argument(*names) = names.last.is_a?(Hash) ? names.pop : {} names.each do |name| arguments.delete_if { |a| a.name == name.to_s } undef_method name, "#{name}=" if [:undefine] end end |
#remove_class_option(*names) ⇒ Object
Removes a previous defined class option.
Paremeters
- names
Class options to be removed
Examples
remove_class_option :foo
remove_class_option :foo, :bar, :baz
227 228 229 230 231 |
# File 'lib/thor/base.rb', line 227 def remove_class_option(*names) names.each do |name| .delete(name) end end |
#remove_task(*names) ⇒ Object
Removes a given task from this Thor class. This is usually done if you are inheriting from another class and don't want it to be available anymore.
By default it only remove the mapping to the task. But you can supply :undefine => true to undefine the method from the class as well.
Parameters
- name<Symbol|String>
The name of the task to be removed
- options
You can give :undefine => true if you want tasks the method to be undefined from the class as well.
281 282 283 284 285 286 287 288 289 |
# File 'lib/thor/base.rb', line 281 def remove_task(*names) = names.last.is_a?(Hash) ? names.pop : {} names.each do |name| tasks.delete(name.to_s) all_tasks.delete(name.to_s) undef_method name if [:undefine] end end |
#start(given_args = ARGV, config = {}) ⇒ Object
Default way to start generators from the command line.
349 350 351 352 353 354 355 356 357 358 |
# File 'lib/thor/base.rb', line 349 def start(given_args=ARGV, config={}) config[:shell] ||= Thor::Base.shell.new yield rescue Thor::Error => e if given_args.include?("--debug") raise e else config[:shell].error e. end end |
#tasks ⇒ Object
Returns the tasks for this Thor class.
Returns
- OrderedHash
An ordered hash with tasks names as keys and Thor::Task objects as values.
254 255 256 |
# File 'lib/thor/base.rb', line 254 def tasks @tasks ||= Thor::CoreExt::OrderedHash.new end |